#include <cmath>
#include <sstream>
#include "ComplexPolar.h"
using namespace std;

#define PI 2*acos(0.0)

ComplexPolar::ComplexPolar(double magnitude, double phase) {
    _mag = magnitude;
    _phase = phase;
}

double ComplexPolar::getReal() {
    return _mag * cos(_phase);
}

double ComplexPolar::getImaginary() {
    return _mag * sin(_phase);
}

double ComplexPolar::getMagnitude() { return _mag; }

double ComplexPolar::getPhase() { return _phase; }

void ComplexPolar::add(ComplexBase* complexPtr) {
    double real, imag;

    real = getReal() + complexPtr->getReal();
    imag = getImaginary() + complexPtr->getImaginary();

    _mag =  sqrt(real*real + imag*imag);
    if (real != 0)
        _phase = atan(imag / real);
    else if (imag > 0)
        _phase = PI / 2;
    else
        _phase = -PI / 2;
}

void ComplexPolar::minus(ComplexBase* complexPtr) {
    double real, imag;

    real = getReal() - complexPtr->getReal();
    imag = getImaginary() - complexPtr->getImaginary();
}

void ComplexPolar::time(ComplexBase* complexPtr) {
    _mag *= complexPtr->getMagnitude();
    _phase += complexPtr->getPhase();
}

string ComplexPolar::toRectangularString() {
    ostringstream os;

    os << "(" << getReal() << ", " << getImaginary() << "i)";
    return os.str();
}

string ComplexPolar::toPolarFormString() {
    double angle;
    ostringstream os;

    angle = getPhase();
    os << getMagnitude() << "(cos "  << angle;
    os <<  " + i sin " << angle << ")";
    return os.str();
}
