#include <cmath>
#include <sstream>
#include "ComplexRectangular.h"
using namespace std;

#define PI 2*acos(0.0)

ComplexRectangular::ComplexRectangular(double real, double imag) {
     _real = real;
     _imag = imag;  
}

double ComplexRectangular::getReal() { return _real; }

double ComplexRectangular::getImaginary() { return _imag; }

double ComplexRectangular::getMagnitude() {
    return sqrt(_real*_real + _imag*_imag);
}

double ComplexRectangular::getPhase() {
    double radian;

    if (_real != 0)
        radian = atan(_imag / _real);
    else if (_imag > 0)
        radian = PI / 2;
    else 
        radian = -PI / 2;
    return radian;
}

void ComplexRectangular::add(ComplexBase* complexPtr) {
   _real = _real + complexPtr->getReal();
   _imag = _imag + complexPtr->getImaginary();
}

void ComplexRectangular::minus(ComplexBase* complexPtr) {
   _real = _real - complexPtr->getReal();
   _imag = _imag - complexPtr->getImaginary();
}

void ComplexRectangular::time(ComplexBase* complexPtr) {
   double realNew, imagNew;

   realNew = _real * complexPtr->getReal() + 
             _imag * complexPtr->getImaginary();
   imagNew  = _real * complexPtr->getImaginary() + 
              _imag * complexPtr->getReal();

   _real = realNew;
   _imag = imagNew;
}

string ComplexRectangular::toRectangularString() {
    ostringstream os;

    os << "(" << getReal() << ", " << getImaginary() << "i)";
    return os.str();
}

string ComplexRectangular::toPolarFormString() {
    double angle;
    ostringstream os;

    angle = getPhase();
    os << getMagnitude() << "(cos "  << angle;
    os <<  " + i sin " << angle << ")";
    return os.str();
}
